////////////////////////
//   BoundedValue.c   //
////////////////////////

#include "BoundedValue.h"
#include "ColorSpace.h"

BoundedValue::BoundedValue() {
    scroll_incr_ = 0.0;
    page_incr_ = 0.0;
}


BoundedValue::BoundedValue(Coord lower, Coord upper) {
    lower_ = lower;
    span_ = upper - lower;
    scroll_incr_ = span_ * 0.04;
    page_incr_ = span_ * 0.4;
    curvalue_ = (lower + upper) * 0.5;
}

BoundedValue::BoundedValue(Coord lower, Coord upper, ColorSpace* cs) {
    lower_ = lower;
    span_ = upper - lower;
    scroll_incr_ = span_ * 0.04;
    page_incr_ = span_ * 0.4;
    curvalue_ = (lower + upper) * 0.5;
    cSpaceType = cs;
}

BoundedValue::~BoundedValue() { }

void BoundedValue::lower_bound(Coord c) { lower_ = c; }
void BoundedValue::upper_bound(Coord c) { span_ = c - lower_; }

void BoundedValue::current_value(Coord value) {
    curvalue_ = value;
    constrain(Dimension_X, curvalue_);
    notify(Dimension_X);
}

void BoundedValue::scroll_incr(Coord c) { scroll_incr_ = c; }
void BoundedValue::page_incr(Coord c) { page_incr_ = c; }

Coord BoundedValue::lower(DimensionName) const { return lower_; }
Coord BoundedValue::upper(DimensionName) const { return lower_ + span_; }
Coord BoundedValue::length(DimensionName) const { return span_; }
Coord BoundedValue::cur_lower(DimensionName) const { return curvalue_; }
Coord BoundedValue::cur_upper(DimensionName) const { return curvalue_; }
Coord BoundedValue::cur_length(DimensionName) const { return 0; }

void BoundedValue::scroll_to(DimensionName d, Coord position) {

    if(cSpaceType->status()){
	Coord p = position;
	constrain(d, p);
	if (p != curvalue_) {
	    curvalue_ = p;
	    // Convert my coordinates into a standard XYZ coornidates first.
		XcmsColor* cmsColor = cSpaceType->convertToXYZ();

	    // indicate that i had set the XYZ values in the conversion space.
	    cSpaceType->setConversionFlag();

	    // Now update the values of XYZ in the ConversionSpace object
	    cSpaceType->conversionSpace()->current_value_X(cmsColor->spec.CIEXYZ.X);
	    cSpaceType->conversionSpace()->current_value_Y(cmsColor->spec.CIEXYZ.Y);
	    cSpaceType->conversionSpace()->current_value_Z(cmsColor->spec.CIEXYZ.Z);

	    cSpaceType->resetConversionFlag();
	    notify_all();
	}
    }
}

void BoundedValue::scroll_forward(DimensionName d) {
    scroll_to(d, curvalue_ + scroll_incr_);
}

void BoundedValue::scroll_backward(DimensionName d) {
    scroll_to(d, curvalue_ - scroll_incr_);
}

void BoundedValue::page_forward(DimensionName d) {
    scroll_to(d, curvalue_ + page_incr_);
}

void BoundedValue::page_backward(DimensionName d) {
    scroll_to(d, curvalue_ - page_incr_);
}


void BoundedValue::setValuator(Valuator* val, DimensionName dim) {
    DimensionName noWarning = dim;
    myValuator = val;
}


Valuator* BoundedValue::valuator() {
    return myValuator;
}

