////////////////////////
//   ColorSpace.cxx   //
////////////////////////

#include "ColorSpace.h"

ColorSpace::ColorSpace(ColorSpaceType spaceType, 
		       ConversionSpace* cSpace,
		       Display* d) {
    cSType = spaceType;
    convSpace = cSpace;
    disp = d;

    iSetConversion=FALSE;

    Coord lower = 0.0;
    Coord upper = 1.0;
    Coord upperZ = 1.5;
    Coord uvYUpper = 0.6;
    Coord LuvUpper =  200.0;
    Coord LuvLower = -200.0;
    Coord xUpper = 0.75; // these values deduced from 
    Coord yUpper = 0.85; // The 1931 CIE Chromotacity Diagram
    Coord LabLower = -100.0;
    Coord HUpper = 360.0;

    wKit = WidgetKit::instance();

    // Adjustable constructor part
    switch(spaceType){
      case RGB:
	upper = (Coord) 0xffff;
	xAdjuster = new BoundedValue(lower, upper, this);
	yAdjuster = new BoundedValue(lower, upper, this);
	zAdjuster = new BoundedValue(lower, upper, this);
	active = wKit->check_box(" RGB Space  ", nil);
	break;
      case RGBi:
	xAdjuster = new BoundedValue(lower, upper, this);
	yAdjuster = new BoundedValue(lower, upper, this);
	zAdjuster = new BoundedValue(lower, upper, this);
	active = wKit->check_box(" RGBi Space ", nil);
	break;
      case XYZ:
	xAdjuster = new BoundedValue(lower, upper, this);
	yAdjuster = new BoundedValue(lower, upper, this);
	zAdjuster = new BoundedValue(lower, upper, this);
	active = wKit->check_box(" XYZ Space  ", nil);
	break;
      case uvY:
	xAdjuster = new BoundedValue(lower, uvYUpper, this);
	yAdjuster = new BoundedValue(lower, uvYUpper, this);
	zAdjuster = new BoundedValue(lower, upper, this);
	active = wKit->check_box(" uvY Space  ", nil);
	break;
      case xyY:
	xAdjuster = new BoundedValue(lower, xUpper, this);
	yAdjuster = new BoundedValue(lower, yUpper, this);
	zAdjuster = new BoundedValue(lower, upper, this);
	active = wKit->check_box(" xyY Space  ", nil);
	break;
      case Lab:
	upper = 100.0;
	xAdjuster = new BoundedValue(lower, upper, this);
	yAdjuster = new BoundedValue(LabLower, upper, this);
	zAdjuster = new BoundedValue(LabLower, upper, this);
	active = wKit->check_box(" Lab Space  ", nil);
	break;
      case Luv:
	upper = 100.0;
	xAdjuster = new BoundedValue(lower, upper, this);
	yAdjuster = new BoundedValue(LuvLower, LuvUpper, this);
	zAdjuster = new BoundedValue(LuvLower, LuvUpper, this);
	active = wKit->check_box(" Luv Space  ", nil);
	break;
      case TekHVC:
	upper = 100.0;
	xAdjuster = new BoundedValue(lower, HUpper, this);
	yAdjuster = new BoundedValue(lower, upper, this);
	zAdjuster = new BoundedValue(lower, upper, this);
	active = wKit->check_box("TekHVC Space", nil);
	break;
      default:
	cout << "Error: Wrong type of colorSpace specified.\n";
	break;
    }

    s = active->state();
    BoundedValue *xtemp = (BoundedValue*)convSpace->XAdjuster();
    BoundedValue *ytemp = (BoundedValue*)convSpace->YAdjuster();
    BoundedValue *ztemp = (BoundedValue*)convSpace->ZAdjuster();

    // attach thyself to the Conversion Space object
    xtemp->attach(Dimension_X, this);
    ytemp->attach(Dimension_X, this);
    ztemp->attach(Dimension_X, this);

    // updating any dimension sets the values of all dimensions.
    update(xtemp->observable(Dimension_X));
}



ColorSpace::~ColorSpace(){
    if(convSpace != nil) {
	((BoundedValue*)convSpace->XAdjuster())->detach(Dimension_X, this);
	((BoundedValue*)convSpace->YAdjuster())->detach(Dimension_X, this);
	((BoundedValue*)convSpace->ZAdjuster())->detach(Dimension_X, this);
    }
}



void ColorSpace::update(Observable*) {

    if(s->test(TelltaleState::is_chosen)) {
	if(iSetConversion == FALSE) {
	    XcmsColor* cmsColor = convertFromXYZ();
	    
	    switch(cSType){
	      case RGB:
		xAdjuster->current_value(cmsColor->spec.RGB.red);
		yAdjuster->current_value(cmsColor->spec.RGB.green);
		zAdjuster->current_value(cmsColor->spec.RGB.blue);
		break;
	      case RGBi:
		xAdjuster->current_value(cmsColor->spec.RGBi.red);
		yAdjuster->current_value(cmsColor->spec.RGBi.green);
		zAdjuster->current_value(cmsColor->spec.RGBi.blue);
		break;
	      case XYZ:
		xAdjuster->current_value(cmsColor->spec.CIEXYZ.X);
		yAdjuster->current_value(cmsColor->spec.CIEXYZ.Y);
		zAdjuster->current_value(cmsColor->spec.CIEXYZ.Z);
		break;
	      case uvY:
		xAdjuster->current_value(cmsColor->spec.CIEuvY.u_prime);
		yAdjuster->current_value(cmsColor->spec.CIEuvY.v_prime);
		zAdjuster->current_value(cmsColor->spec.CIEuvY.Y);
		break;
	      case xyY:
		xAdjuster->current_value(cmsColor->spec.CIExyY.x);
		yAdjuster->current_value(cmsColor->spec.CIExyY.y);
		zAdjuster->current_value(cmsColor->spec.CIExyY.Y);
		break;
	      case Lab:
		xAdjuster->current_value(cmsColor->spec.CIELab.L_star);
		yAdjuster->current_value(cmsColor->spec.CIELab.a_star);
		zAdjuster->current_value(cmsColor->spec.CIELab.b_star);
		break;
	      case Luv:
		xAdjuster->current_value(cmsColor->spec.CIELuv.L_star);
		yAdjuster->current_value(cmsColor->spec.CIELuv.u_star);
		zAdjuster->current_value(cmsColor->spec.CIELuv.v_star);
		break;
	      case TekHVC:
		xAdjuster->current_value(cmsColor->spec.TekHVC.H);
		yAdjuster->current_value(cmsColor->spec.TekHVC.V);
		zAdjuster->current_value(cmsColor->spec.TekHVC.C);
		break;
	      default:
		cout << "Error: Wrong type of Color Space specified.\n";
		break;
	    }
	    xAdjuster->notify(Dimension_X);
	    yAdjuster->notify(Dimension_X);
	    zAdjuster->notify(Dimension_X);
	}
    } else {
	xAdjuster->current_value(0.0);
	yAdjuster->current_value(0.0);
	zAdjuster->current_value(0.0);
	xAdjuster->notify(Dimension_X);
	yAdjuster->notify(Dimension_X);
	zAdjuster->notify(Dimension_X);
    }
}


XcmsColor* ColorSpace::convertToXYZ() {
    
    XDisplay* Xdisp = disp->rep()->display_;

    XcmsCCC defCCC = XcmsDefaultCCC(Xdisp, DefaultScreen(Xdisp));


    switch(cSType){
      case RGB:
	xcColor.spec.RGB.red   = (short)xAdjuster->cur_lower(Dimension_X);
	xcColor.spec.RGB.green = (short)yAdjuster->cur_lower(Dimension_X);
	xcColor.spec.RGB.blue  = (short)zAdjuster->cur_lower(Dimension_X);
	xcColor.format = XcmsRGBFormat;
	break;
      case RGBi:
	xcColor.spec.RGBi.red   = xAdjuster->cur_lower(Dimension_X);
	xcColor.spec.RGBi.green = yAdjuster->cur_lower(Dimension_X);
	xcColor.spec.RGBi.blue  = zAdjuster->cur_lower(Dimension_X);
	xcColor.format = XcmsRGBiFormat;
	break;
      case XYZ:
	xcColor.spec.CIEXYZ.X = xAdjuster->cur_lower(Dimension_X);
	xcColor.spec.CIEXYZ.Y = yAdjuster->cur_lower(Dimension_X);
	xcColor.spec.CIEXYZ.Z = zAdjuster->cur_lower(Dimension_X);
	xcColor.format = XcmsCIEXYZFormat;
	break;
      case uvY:
	xcColor.spec.CIEuvY.u_prime = xAdjuster->cur_lower(Dimension_X);
	xcColor.spec.CIEuvY.v_prime = yAdjuster->cur_lower(Dimension_X);
	xcColor.spec.CIEuvY.Y       = zAdjuster->cur_lower(Dimension_X);
	xcColor.format = XcmsCIEuvYFormat;
	break;
      case xyY:
	xcColor.spec.CIExyY.x = xAdjuster->cur_lower(Dimension_X);
	xcColor.spec.CIExyY.y = yAdjuster->cur_lower(Dimension_X);
	xcColor.spec.CIExyY.Y = zAdjuster->cur_lower(Dimension_X);
	xcColor.format = XcmsCIExyYFormat;
	break;
      case Lab:
	xcColor.spec.CIELab.L_star = xAdjuster->cur_lower(Dimension_X);
	xcColor.spec.CIELab.a_star = yAdjuster->cur_lower(Dimension_X);
	xcColor.spec.CIELab.b_star = zAdjuster->cur_lower(Dimension_X);
	xcColor.format = XcmsCIELabFormat;
	break;
      case Luv:
	xcColor.spec.CIELuv.L_star = xAdjuster->cur_lower(Dimension_X);
	xcColor.spec.CIELuv.u_star = yAdjuster->cur_lower(Dimension_X);
	xcColor.spec.CIELuv.v_star = zAdjuster->cur_lower(Dimension_X);
	xcColor.format = XcmsCIELuvFormat;
	break;
      case TekHVC:
	xcColor.spec.TekHVC.H = xAdjuster->cur_lower(Dimension_X);
	xcColor.spec.TekHVC.V = yAdjuster->cur_lower(Dimension_X);
	xcColor.spec.TekHVC.C = zAdjuster->cur_lower(Dimension_X);
	xcColor.format = XcmsTekHVCFormat;
	break;
      default:
	cout << "Error: Wrong type of Color Space specified.\n";
	break;
    }


    if( XcmsFailure == 
           XcmsConvertColors(
			     defCCC, &xcColor, 1, XcmsCIEXYZFormat, nil
			     )
       ) 
	cout << "Error: Color conversion to XYZ failed.\n";

    return &xcColor;
}




XcmsColor* ColorSpace::convertFromXYZ(){

    XDisplay* Xdisp = disp->rep()->display_;

    XcmsCCC defCCC = XcmsDefaultCCC(Xdisp, DefaultScreen(Xdisp));

    xcColor.spec.CIEXYZ.X = convSpace->cur_lower_X();
    xcColor.spec.CIEXYZ.Y = convSpace->cur_lower_Y();
    xcColor.spec.CIEXYZ.Z = convSpace->cur_lower_Z();

    XcmsColorFormat colFormat;

    switch(cSType){
      case RGB:
	colFormat = XcmsRGBFormat;
	break;
      case RGBi:
	colFormat = XcmsRGBiFormat;
	break;
      case XYZ:
	colFormat = XcmsCIEXYZFormat;
	break;
      case uvY:
	colFormat = XcmsCIEuvYFormat;
	break;
      case xyY:
	colFormat = XcmsCIExyYFormat;
	break;
      case Lab:
	colFormat = XcmsCIELabFormat;
	break;
      case Luv:
	colFormat = XcmsCIELuvFormat;
	break;
      case TekHVC:
	colFormat = XcmsTekHVCFormat;
	break;
      default:
	cout << "Error: Wrong type of Color Space specified.\n";
	break;
    }

    xcColor.format = XcmsCIEXYZFormat;

    if ( XcmsFailure == 
           XcmsConvertColors(
			     defCCC, &xcColor, 1, colFormat, nil
			     )
       ) 
	cout << "Error: Color converstion from XYZ failed.\n";

    return &xcColor;
}


BoundedValue* ColorSpace::XAdjuster() {
    return xAdjuster;
}


BoundedValue* ColorSpace::YAdjuster() {
    return yAdjuster;
}


BoundedValue* ColorSpace::ZAdjuster() {
    return zAdjuster;
}
    

ConversionSpace* ColorSpace::conversionSpace() {
    return convSpace;
}


void ColorSpace::setConversionFlag() {
    iSetConversion = TRUE;
}


void ColorSpace::resetConversionFlag() {
    iSetConversion = FALSE;
}


boolean ColorSpace::status() {
    return s->test(TelltaleState::is_chosen);
}


Glyph* ColorSpace::presentation(LayoutKit& l) {
    Glyph *xCoordLabel, *yCoordLabel, *zCoordLabel;
    switch(cSType) {
      case RGB:
	xCoordLabel = wKit->fancy_label(" R ");
	yCoordLabel = wKit->fancy_label(" G ");
	zCoordLabel = wKit->fancy_label(" B ");
	break;
      case RGBi:
	xCoordLabel = wKit->fancy_label(" R ");
	yCoordLabel = wKit->fancy_label(" G ");
	zCoordLabel = wKit->fancy_label(" B ");
	break;
      case XYZ:
	xCoordLabel = wKit->fancy_label(" X ");
	yCoordLabel = wKit->fancy_label(" Y ");
	zCoordLabel = wKit->fancy_label(" Z ");
	break;
      case uvY:
	xCoordLabel = wKit->fancy_label(" u ");
	yCoordLabel = wKit->fancy_label(" v ");
	zCoordLabel = wKit->fancy_label(" Y ");
	break;
      case xyY:
	xCoordLabel = wKit->fancy_label(" x ");
	yCoordLabel = wKit->fancy_label(" y ");
	zCoordLabel = wKit->fancy_label(" Y ");
	break;
      case Lab:
	xCoordLabel = wKit->fancy_label(" L ");
	yCoordLabel = wKit->fancy_label(" a ");
	zCoordLabel = wKit->fancy_label(" b ");
	break;
      case Luv:
	xCoordLabel = wKit->fancy_label(" L ");
	yCoordLabel = wKit->fancy_label(" u ");
	zCoordLabel = wKit->fancy_label(" v ");
	break;
      case TekHVC:
	xCoordLabel = wKit->fancy_label(" H ");
	yCoordLabel = wKit->fancy_label(" V ");
	zCoordLabel = wKit->fancy_label(" C ");	 
	break;
      default:
	cout << "Error: Wrong type of Color Space specified.\n";
	break;
    }

    Valuator* xv = xAdjuster->valuator();
    Valuator* yv = yAdjuster->valuator();
    Valuator* zv = zAdjuster->valuator();
    
    return (
	    l.vbox(
		   l.hbox(
			  l.vbox(
				 l.hcenter(active),
			  ),
			  l.vbox(
				 l.hbox(
					XScrollBar(),
					xCoordLabel,
					xv
		                 ),
				 l.hbox(
					YScrollBar(),
					yCoordLabel,
					yv
			         ),
				 l.hbox(
					ZScrollBar(),
					zCoordLabel,
					zv
	                         )
                          )
		   ),
		   l.vglue(15.0, 15.0, 5.0)
           )
     );
}



Glyph* ColorSpace::XScrollBar() {
	return wKit->scroll_bar_look(Dimension_X, XAdjuster());
}


Glyph* ColorSpace::YScrollBar() {
	return wKit->scroll_bar_look(Dimension_X, YAdjuster());
}


Glyph* ColorSpace::ZScrollBar() {
	return wKit->scroll_bar_look(Dimension_X, ZAdjuster());
}


void ColorSpace::setValuator(Valuator* val, DimensionName dim) {
    switch(dim) {
      case Dimension_X:
	xAdjuster->setValuator(val,Dimension_X);
	break;
      case Dimension_Y:
	yAdjuster->setValuator(val,Dimension_X);
	break;
      case Dimension_Z:
	zAdjuster->setValuator(val,Dimension_X);
	break;
      default:
	cout << "Error: Wrong type of dimension specified.\n";
	break;
    }
}
    
