//////////////////////
//   ColorSpace.h   //
//////////////////////

// ColorSpace class represents color in some specified space.
// It acts both as an Observer as well as an Adjustable
// As an Observer, it monitors ConversionSpace and updates itself
// as soon as there is a change in the ConversionSpace coordinates.
// When acting as an Adjustable, its values in the three dimensions are
// changed by the change in the scrollbars to which they are attached.

#ifndef _ColorSpace_h
#define _ColorSpace_h 1


#include <IV-look/kit.h>
#include <IV-look/menu.h>
#include <InterViews/observe.h>
#include <InterViews/display.h>
#include <InterViews/patch.h>
#include <InterViews/session.h>
#include <InterViews/window.h>
#include <InterViews/layout.h>
#include <InterViews/background.h>
#include <InterViews/color.h>
#include <OS/string.h>
#include <IV-look/bevel.h>
#include <IV-look/mf_kit.h>
#include <IV-X11/xdisplay.h>
#include "ConversionSpace.h"
#include "BoundedValue.h"
#include "Valuator.h"

extern "C" {
#include <IV-X11/Xdefs.h>
#include <X11/Xcms.h>
#include <IV-X11/Xundefs.h>
double _XcmsSquareRoot(double);
double _XcmsArcTangent(double);
}

enum ColorSpaceType {RGB, RGBi, XYZ, uvY, xyY, Lab, Luv, TekHVC} ;



class ColorSpace : public Observer {
  public:
    ColorSpace(ColorSpaceType, ConversionSpace*, Display*);
    ~ColorSpace();

    // Observer Inteface part
    virtual void update(Observable*);


    XcmsColor* convertToXYZ();
    XcmsColor* convertFromXYZ();

    BoundedValue* XAdjuster();
    BoundedValue* YAdjuster();
    BoundedValue* ZAdjuster();
    ConversionSpace* conversionSpace();
    void setConversionFlag();
    void resetConversionFlag();

    void setValuator(Valuator*, DimensionName);
    Glyph* presentation(LayoutKit&);

    boolean status();
    Button* active;

    Glyph* XScrollBar();
    Glyph* YScrollBar();
    Glyph* ZScrollBar();

  private:
    BoundedValue *xAdjuster, *yAdjuster, *zAdjuster;
    ConversionSpace* convSpace;
    ColorSpaceType cSType;
    Display* disp;
    WidgetKit* wKit;

    Boolean updated;
    Boolean initializer;

    // convert TO and FROM XYZ coordinate 
    XcmsColor xcColor;
    Boolean iSetConversion;      // a flag to indicate if i am the initiator
	                         // of a change
    TelltaleState* s;
};

#endif

