///////////////////////
//   ColorViewer.h   //
///////////////////////

// This class shows the color. It behaves as an Observer as well as a Patch.
// It obeserves the changes in the RGB ColorSpace and creates a Color
// depending on the current values of R, G and B. This color is then 
// shown on the screen.

// Borrowed from Dave Sternlicht's ColorViewer class

#ifndef _ColorViewer_h
#define _ColorViewer_h 1

#include <InterViews/color.h>
#include <InterViews/display.h>
#include <InterViews/patch.h>
#include <InterViews/observe.h>
#include <InterViews/style.h>
#include "ColorSpace.h"


class ColorViewer : public Patch, public Observer{
 public:
    ColorViewer(ColorSpace*, Style*, Display*);
    ~ColorViewer();

    virtual void allocate(Canvas*, const Allocation&, Extension&);
    virtual void draw(Canvas*, const Allocation&) const;

    virtual void update(Observable*);
    virtual void disconnect(Observable*);

 private:
    ColorSpace* clSpace;
    Color* color;
    Display* disp;
};



#endif
