///////////////////////////
//   Valuator.cxx        //
///////////////////////////

#include "Valuator.h"
#include "ColorSpace.h"

declareFieldEditorCallback(Valuator)
implementFieldEditorCallback(Valuator)

Valuator::Valuator(ColorSpace* cs, DimensionName d, Style* style) : MonoGlyph(nil) {
    Style* s = new Style(style);
    s->alias("Valuator");
    switch(d) {
      case Dimension_X:
	clSpaceAdjuster_ = cs->XAdjuster();
	break;
      case Dimension_Y:
	clSpaceAdjuster_ = cs->YAdjuster();
	break;
      case Dimension_Z:
	clSpaceAdjuster_ = cs->ZAdjuster();
	break;
      default:
	cout << "Error: Wrong type of dimension specified.\n";
	break;
    }
    dimension_ = Dimension_X;
    clSpaceAdjuster_->attach(dimension_, this);
    editor_ = DialogKit::instance()->field_editor(
	"100.00", s,
	new FieldEditorCallback(Valuator)(
            this, &Valuator::accept_editor, &Valuator::cancel_editor
        )
    );
    body(editor_);
    update(clSpaceAdjuster_->observable(dimension_));
}

Valuator::~Valuator() {
    if (clSpaceAdjuster_ != nil) {
	clSpaceAdjuster_->detach(dimension_, this);
    }
}


InputHandler* Valuator::focusable() const {
    return editor_;
}


void Valuator::update(Observable*) {
    Coord v = clSpaceAdjuster_->cur_lower(dimension_);
    char buf[20];
    sprintf(buf, "%.2f", v);
    editor_->field(buf);
}


void Valuator::disconnect(Observable*) {
    clSpaceAdjuster_ = nil;
}

void Valuator::accept_editor(FieldEditor*) {
    Coord v;
    const String& value = *editor_->text();
    if (value.convert(v)) {
        clSpaceAdjuster_->current_value(v);
    }
}

void Valuator::cancel_editor(FieldEditor*) {
    update(clSpaceAdjuster_->observable(Dimension_X));
}
